@@ -3,34 +3,33 @@
     unit test for the filters
     ~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    Missing tests:
-
-    -   wordcount
-    -   rst
-    -   markdown
-    -   textile
-
-    :copyright: 2007 by Armin Ronacher.
+    :copyright: 2008 by Armin Ronacher.
     :license: BSD, see LICENSE for more details.
 """
+from jinja2 import Markup, Environment
+
 
 CAPITALIZE = '''{{ "foo bar"|capitalize }}'''
-CAPTURE = '''{{ "foo"|capture('bar') }}|{{ bar }}'''
 CENTER = '''{{ "foo"|center(9) }}'''
 DEFAULT = '''{{ missing|default("no") }}|{{ false|default('no') }}|\
 {{ false|default('no', true) }}|{{ given|default("no") }}'''
 DICTSORT = '''{{ foo|dictsort }}|\
 {{ foo|dictsort(true) }}|\
 {{ foo|dictsort(false, 'value') }}'''
-BATCH = '''{{ foo|batch(3) }}|{{ foo|batch(3, 'X') }}'''
-SLICE = '''{{ foo|slice(3) }}|{{ foo|slice(3, 'X') }}'''
-ESCAPE = '''{{ '<">&'|escape }}|{{ '<">&'|escape(true) }}'''
+BATCH = '''{{ foo|batch(3)|list }}|{{ foo|batch(3, 'X')|list }}'''
+SLICE = '''{{ foo|slice(3)|list }}|{{ foo|slice(3, 'X')|list }}'''
+ESCAPE = '''{{ '<">&'|escape }}'''
 STRIPTAGS = '''{{ foo|striptags }}'''
 FILESIZEFORMAT = '{{ 100|filesizeformat }}|\
 {{ 1000|filesizeformat }}|\
 {{ 1000000|filesizeformat }}|\
 {{ 1000000000|filesizeformat }}|\
-{{ 1000000000000|filesizeformat }}'
+{{ 1000000000000|filesizeformat }}|\
+{{ 100|filesizeformat(true) }}|\
+{{ 1000|filesizeformat(true) }}|\
+{{ 1000000|filesizeformat(true) }}|\
+{{ 1000000000|filesizeformat(true) }}|\
+{{ 1000000000000|filesizeformat(true) }}'
 FIRST = '''{{ foo|first }}'''
 FLOAT = '''{{ "42"|float }}|{{ "ajsghasjgd"|float }}|{{ "32.32"|float }}'''
 FORMAT = '''{{ "%s|%s"|format("a", "b") }}'''
@@ -42,7 +41,7 @@ LENGTH = '''{{ "hello world"|length }}'''
 LOWER = '''{{ "FOO"|lower }}'''
 PPRINT = '''{{ data|pprint }}'''
 RANDOM = '''{{ seq|random }}'''
-REVERSE = '''{{ "foobar"|reverse }}|{{ [1, 2, 3]|reverse }}'''
+REVERSE = '''{{ "foobar"|reverse|join }}|{{ [1, 2, 3]|reverse|list }}'''
 STRING = '''{{ range(10)|string }}'''
 TITLE = '''{{ "foo bar"|title }}'''
 TRIM = '''{{ "      foo       "|trim }}'''
@@ -50,7 +49,6 @@ TRUNCATE = '''{{ data|truncate(15, true, ">>>") }}|\
 {{ data|truncate(15, false, ">>>") }}|\
 {{ smalldata|truncate(15) }}'''
 UPPER = '''{{ "foo"|upper }}'''
-URLENCODE = '''{{ "f#b"|urlencode }}'''
 URLIZE = '''{{ "foo http://www.example.com/ bar"|urlize }}'''
 WORDCOUNT = '''{{ "foo bar baz"|wordcount }}'''
 BLOCK = '''{% filter lower|escape %}<HEHE>{% endfilter %}'''
@@ -62,24 +60,20 @@ ROUND = '''{{ 2.7|round }}|{{ 2.1|round }}|\
 XMLATTR = '''{{ {'foo': 42, 'bar': 23, 'fish': none,
 'spam': missing, 'blub:blub': '<?>'}|xmlattr }}'''
 SORT = '''{{ [2, 3, 1]|sort }}|{{ [2, 3, 1]|sort(true) }}'''
-GROUPBY = '''{{ [{'foo': 1, 'bar': 2},
+GROUPBY = '''{% for grouper, list in [{'foo': 1, 'bar': 2},
                  {'foo': 2, 'bar': 3},
                  {'foo': 1, 'bar': 1},
-                 {'foo': 3, 'bar': 4}]|groupby('foo') }}'''
+                 {'foo': 3, 'bar': 4}]|groupby('foo') -%}
+{{ grouper }}: {{ list|join(', ') }}
+{% endfor %}'''
 FILTERTAG = '''{% filter upper|replace('FOO', 'foo') %}foobar{% endfilter %}'''
 
 
-
 def test_capitalize(env):
     tmpl = env.from_string(CAPITALIZE)
     assert tmpl.render() == 'Foo bar'
 
 
-def test_capture(env):
-    tmpl = env.from_string(CAPTURE)
-    assert tmpl.render() == 'foo|foo'
-
-
 def test_center(env):
     tmpl = env.from_string(CENTER)
     assert tmpl.render() == '   foo   '
@@ -115,7 +109,7 @@ def test_slice(env):
 def test_escape(env):
     tmpl = env.from_string(ESCAPE)
     out = tmpl.render()
-    assert out == '&lt;"&gt;&amp;|&lt;&quot;&gt;&amp;'
+    assert out == '&lt;&#34;&gt;&amp;'
 
 
 def test_striptags(env):
@@ -129,7 +123,10 @@ def test_striptags(env):
 def test_filesizeformat(env):
     tmpl = env.from_string(FILESIZEFORMAT)
     out = tmpl.render()
-    assert out == '100 Bytes|1000 Bytes|976.6 KB|953.7 MB|931.3 GB'
+    assert out == (
+        '100 Bytes|1.0 KB|1.0 MB|1.0 GB|1000.0 GB|'
+        '100 Bytes|1000 Bytes|976.6 KiB|953.7 MiB|931.3 GiB'
+    )
 
 
 def test_first(env):
@@ -169,6 +166,10 @@ def test_join(env):
     out = tmpl.render()
     assert out == '1|2|3'
 
+    env2 = Environment(autoescape=True)
+    tmpl = env2.from_string('{{ ["<foo>", "<span>foo</span>"|safe]|join }}')
+    assert tmpl.render() == '&lt;foo&gt;<span>foo</span>'
+
 
 def test_last(env):
     tmpl = env.from_string(LAST)
@@ -191,7 +192,7 @@ def test_lower(env):
 def test_pprint(env):
     from pprint import pformat
     tmpl = env.from_string(PPRINT)
-    data = range(10000)
+    data = range(1000)
     assert tmpl.render(data=data) == pformat(data)
 
 
@@ -209,7 +210,7 @@ def test_reverse(env):
 
 def test_string(env):
     tmpl = env.from_string(STRING)
-    assert tmpl.render(foo=range(10)) == str(range(10))
+    assert tmpl.render(foo=range(10)) == unicode(xrange(10))
 
 
 def test_title(env):
@@ -234,11 +235,6 @@ def test_upper(env):
     assert tmpl.render() == 'FOO'
 
 
-def test_urlencode(env):
-    tmpl = env.from_string(URLENCODE)
-    assert tmpl.render() == 'f%23b'
-
-
 def test_urlize(env):
     tmpl = env.from_string(URLIZE)
     assert tmpl.render() == 'foo <a href="http://www.example.com/">'\
@@ -291,13 +287,40 @@ def test_sort(env):
 
 def test_groupby(env):
     tmpl = env.from_string(GROUPBY)
-    assert tmpl.render() == (
-        "[{'list': [{'foo': 1, 'bar': 2}, {'foo': 1, 'bar': 1}], "
-        "'grouper': 1}, {'list': [{'foo': 2, 'bar': 3}], 'grouper': 2}, "
-        "{'list': [{'foo': 3, 'bar': 4}], 'grouper': 3}]"
-    )
+    assert tmpl.render().splitlines() == [
+        "1: {'foo': 1, 'bar': 2}, {'foo': 1, 'bar': 1}",
+        "2: {'foo': 2, 'bar': 3}",
+        "3: {'foo': 3, 'bar': 4}"
+    ]
 
 
 def test_filtertag(env):
     tmpl = env.from_string(FILTERTAG)
     assert tmpl.render() == 'fooBAR'
+
+
+def test_replace():
+    env = Environment()
+    tmpl = env.from_string('{{ string|replace("o", 42) }}')
+    assert tmpl.render(string='<foo>') == '<f4242>'
+
+    env = Environment(autoescape=True)
+    tmpl = env.from_string('{{ string|replace("o", 42) }}')
+    assert tmpl.render(string='<foo>') == '&lt;f4242&gt;'
+    tmpl = env.from_string('{{ string|replace("<", 42) }}')
+    assert tmpl.render(string='<foo>') == '42foo&gt;'
+    tmpl = env.from_string('{{ string|replace("o", ">x<") }}')
+    assert tmpl.render(string=Markup('foo')) == 'f&gt;x&lt;&gt;x&lt;'
+
+
+def test_forceescape(env):
+    tmpl = env.from_string('{{ x|forceescape }}')
+    assert tmpl.render(x=Markup('<div />')) == u'&lt;div /&gt;'
+
+
+def test_safe():
+    env = Environment(autoescape=True)
+    tmpl = env.from_string('{{ "<div>foo</div>"|safe }}')
+    assert tmpl.render() == '<div>foo</div>'
+    tmpl = env.from_string('{{ "<div>foo</div>" }}')
+    assert tmpl.render() == '&lt;div&gt;foo&lt;/div&gt;'
