@@ -6,15 +6,15 @@
     :copyright: 2007 by Armin Ronacher.
     :license: BSD, see LICENSE for more details.
 """
-from jinja import Environment, DictLoader
-from jinja.exceptions import TemplateSyntaxError
+from py.test import raises
+from jinja2 import Environment, DictLoader
+from jinja2.exceptions import TemplateSyntaxError, UndefinedError
 
 
 CALL = '''{{ foo('a', c='d', e='f', *['b'], **{'g': 'h'}) }}'''
 SLICING = '''{{ [1, 2, 3][:] }}|{{ [1, 2, 3][::-1] }}'''
 ATTR = '''{{ foo.bar }}|{{ foo['bar'] }}'''
 SUBSCRIPT = '''{{ foo[0] }}|{{ foo[-1] }}'''
-KEYATTR = '''{{ {'items': 'foo'}.items }}|{{ {}.items() }}'''
 TUPLE = '''{{ () }}|{{ (1,) }}|{{ (1, 2) }}'''
 MATH = '''{{ (1 + 1 * 2) - 3 / 2 }}|{{ 2**3 }}'''
 DIV = '''{{ 3 // 2 }}|{{ 3 / 2 }}|{{ 3 % 2 }}'''
@@ -22,13 +22,12 @@ UNARY = '''{{ +3 }}|{{ -3 }}'''
 CONCAT = '''{{ [1, 2] ~ 'foo' }}'''
 COMPARE = '''{{ 1 > 0 }}|{{ 1 >= 1 }}|{{ 2 < 3 }}|{{ 2 == 2 }}|{{ 1 <= 1 }}'''
 INOP = '''{{ 1 in [1, 2, 3] }}|{{ 1 not in [1, 2, 3] }}'''
-LITERALS = '''{{ [] }}|{{ {} }}|{{ () }}|{{ '' }}|{{ @() }}'''
+LITERALS = '''{{ [] }}|{{ {} }}|{{ () }}'''
 BOOL = '''{{ true and false }}|{{ false or true }}|{{ not false }}'''
 GROUPING = '''{{ (true and false) or (false and true) and not false }}'''
 CONDEXPR = '''{{ 0 if true else 1 }}'''
 DJANGOATTR = '''{{ [1, 2, 3].0 }}'''
 FILTERPRIORITY = '''{{ "foo"|upper + "bar"|upper }}'''
-REGEX = '''{{ @/\S+/.findall('foo bar baz') }}'''
 TUPLETEMPLATES = [
     '{{ () }}',
     '{{ (1, 2) }}',
@@ -37,16 +36,13 @@ TUPLETEMPLATES = [
     '{{ 1, 2 }}',
     '{% for foo, bar in seq %}...{% endfor %}',
     '{% for x in foo, bar %}...{% endfor %}',
-    '{% for x in foo, %}...{% endfor %}',
-    '{% for x in foo, recursive %}...{% endfor %}',
-    '{% for x in foo, bar recursive %}...{% endfor %}',
-    '{% for x, in foo, recursive %}...{% endfor %}'
+    '{% for x in foo, %}...{% endfor %}'
 ]
-TRAILINGCOMMA = '''{{ (1, 2,) }}|{{ [1, 2,] }}|{{ {1: 2,} }}|{{ @(1, 2,) }}'''
+TRAILINGCOMMA = '''{{ (1, 2,) }}|{{ [1, 2,] }}|{{ {1: 2,} }}'''
 
 
 def test_call():
-    from jinja import Environment
+    from jinja2 import Environment
     env = Environment()
     env.globals['foo'] = lambda a, b, c, e, g: a + b + c + e + g
     tmpl = env.from_string(CALL)
@@ -68,11 +64,6 @@ def test_subscript(env):
     assert tmpl.render(foo=[0, 1, 2]) == '0|2'
 
 
-def test_keyattr(env):
-    tmpl = env.from_string(KEYATTR)
-    assert tmpl.render() == 'foo|[]'
-
-
 def test_tuple(env):
     tmpl = env.from_string(TUPLE)
     assert tmpl.render() == '()|(1,)|(1, 2)'
@@ -110,7 +101,7 @@ def test_inop(env):
 
 def test_literals(env):
     tmpl = env.from_string(LITERALS)
-    assert tmpl.render() == '[]|{}|()||set([])'
+    assert tmpl.render().lower() == '[]|{}|()'
 
 
 def test_bool(env):
@@ -133,6 +124,14 @@ def test_conditional_expression(env):
     assert tmpl.render() == '0'
 
 
+def test_short_conditional_expression(env):
+    tmpl = env.from_string('<{{ 1 if false }}>')
+    assert tmpl.render() == '<>'
+
+    tmpl = env.from_string('<{{ (1 if false).bar }}>')
+    raises(UndefinedError, tmpl.render)
+
+
 def test_filter_priority(env):
     tmpl = env.from_string(FILTERPRIORITY)
     assert tmpl.render() == 'FOOBAR'
@@ -153,46 +152,40 @@ def test_function_calls(env):
     ]
     for should_fail, sig in tests:
         if should_fail:
-            try:
-                print env.from_string('{{ foo(%s) }}' % sig)
-            except TemplateSyntaxError:
-                continue
-            assert False, 'expected syntax error'
+            raises(TemplateSyntaxError, env.from_string, '{{ foo(%s) }}' % sig)
         else:
             env.from_string('foo(%s)' % sig)
 
 
-def test_regex(env):
-    tmpl = env.from_string(REGEX)
-    assert tmpl.render() == "['foo', 'bar', 'baz']"
-
-
 def test_tuple_expr(env):
     for tmpl in TUPLETEMPLATES:
+        print tmpl
         assert env.from_string(tmpl)
 
 
 def test_trailing_comma(env):
     tmpl = env.from_string(TRAILINGCOMMA)
-    assert tmpl.render() == '(1, 2)|[1, 2]|{1: 2}|set([1, 2])'
+    assert tmpl.render().lower() == '(1, 2)|[1, 2]|{1: 2}'
 
 
-def test_extends_position():
-    env = Environment(loader=DictLoader({
-        'empty': '[{% block empty %}{% endblock %}]'
-    }))
-    tests = [
-        ('{% extends "empty" %}', '[!]'),
-        ('  {% extends "empty" %}', '[!]'),
-        ('  !\n', '  !\n!'),
-        ('{# foo #}  {% extends "empty" %}', '[!]'),
-        ('{% set foo = "blub" %}{% extends "empty" %}', None)
-    ]
+def test_block_end_name(env):
+    env.from_string('{% block foo %}...{% endblock foo %}')
+    raises(TemplateSyntaxError, env.from_string, '{% block x %}{% endblock y %}')
 
-    for tmpl, expected_output in tests:
-        try:
-            tmpl = env.from_string(tmpl + '{% block empty %}!{% endblock %}')
-        except TemplateSyntaxError:
-            assert expected_output is None, 'got syntax error'
-        else:
-            assert expected_output == tmpl.render()
+
+def test_contant_casing(env):
+    for const in True, False, None:
+        tmpl = env.from_string('{{ %s }}|{{ %s }}|{{ %s }}' % (
+            str(const), str(const).lower(), str(const).upper()
+        ))
+        assert tmpl.render() == '%s|%s|' % (const, const)
+
+
+def test_test_chaining(env):
+    raises(TemplateSyntaxError, env.from_string, '{{ foo is string is sequence }}')
+    env.from_string('{{ 42 is string or 42 is number }}').render() == 'True'
+
+
+def test_string_concatenation(env):
+    tmpl = env.from_string('{{ "foo" "bar" "baz" }}')
+    assert tmpl.render() == 'foobarbaz'
